# Copyright 2010 New Relic, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

import pytest
from testing_support.mock_external_http_server import MockExternalHTTPServer

from newrelic.common.package_version_utils import get_package_version_tuple

# This defines an external server test apps can make requests to instead of
# the real Google Gemini backend. This provides 3 features:
#
# 1) This removes dependencies on external websites.
# 2) Provides a better mechanism for making an external call in a test app than
#    simple calling another endpoint the test app makes available because this
#    server will not be instrumented meaning we don't have to sort through
#    transactions to separate the ones created in the test app and the ones
#    created by an external call.
# 3) This app runs on a separate thread meaning it won't block the test app.

RESPONSES = {
    "Invalid API key.": [
        {
            "content-type": "application/json",
            "x-goog-api-key": "GEMINI_API_KEY",
            "x-goog-api-client": "google-genai-sdk/1.8.0 gl-python/3.11.4",
        },
        400,
        {
            "error": {
                "code": 400,
                "message": "API key not valid. Please pass a valid API key.",
                "status": "INVALID_ARGUMENT",
                "details": [
                    {
                        "@type": "type.googleapis.com/google.rpc.ErrorInfo",
                        "reason": "API_KEY_INVALID",
                        "domain": "googleapis.com",
                        "metadata": {"service": "generativelanguage.googleapis.com"},
                    },
                    {
                        "@type": "type.googleapis.com/google.rpc.LocalizedMessage",
                        "locale": "en-US",
                        "message": "API key not valid. Please pass a valid API key.",
                    },
                ],
            }
        },
    ],
    "Model does not exist.": [
        {
            "content-type": "application/json",
            "x-goog-api-key": "GEMINI_API_KEY",
            "x-goog-api-client": "google-genai-sdk/1.8.0 gl-python/3.11.4",
        },
        404,
        {
            "error": {
                "code": 404,
                "message": "models/does-not-exist is not found for API version v1beta, or is not supported for generateContent. Call ListModels to see the list of available models and their supported methods.",
                "status": "NOT_FOUND",
            }
        },
    ],
    "Embedded: Model does not exist.": [
        {
            "content-type": "application/json",
            "x-goog-api-key": "GEMINI_API_KEY",
            "x-goog-api-client": "google-genai-sdk/1.8.0 gl-python/3.11.4",
        },
        404,
        {
            "error": {
                "code": 404,
                "message": "models/does-not-exist is not found for API version v1beta, or is not supported for embedContent. Call ListModels to see the list of available models and their supported methods.",
                "status": "NOT_FOUND",
            }
        },
    ],
    "How many letters are in the word Python?": [
        {
            "content-type": "application/json",
            "x-goog-api-key": "GEMINI_API_KEY",
            "x-goog-api-client": "google-genai-sdk/1.8.0 gl-python/3.11.4",
        },
        200,
        {
            "candidates": [
                {
                    "content": {
                        "parts": [{"text": 'There are **6** letters in the word "Python".\n'}],
                        "role": "model",
                    },
                    "finishReason": "STOP",
                    "avgLogprobs": -0.21492499571580154,
                }
            ],
            "usageMetadata": {
                "promptTokenCount": 9,
                "candidatesTokenCount": 13,
                "totalTokenCount": 22,
                "promptTokensDetails": [{"modality": "TEXT", "tokenCount": 9}],
                "candidatesTokensDetails": [{"modality": "TEXT", "tokenCount": 13}],
            },
            "modelVersion": "gemini-2.0-flash",
        },
    ],
    "This is an embedding test.": [
        {
            "content-type": "application/json",
            "x-goog-api-key": "GEMINI_API_KEY",
            "x-goog-api-client": "google-genai-sdk/1.8.0 gl-python/3.11.4",
        },
        200,
        {
            "embedding": {
                "values": [
                    0.0042326464,
                    0.029004563,
                    -0.07853497,
                    0.015845867,
                    0.060363144,
                    0.008264836,
                    0.08827857,
                    0.051790033,
                    -0.010739345,
                    0.0097664865,
                    -0.0030682562,
                    0.07532256,
                    0.06018427,
                    -0.018718004,
                    0.0030775273,
                    -0.06404092,
                    0.01164792,
                    0.010430611,
                    -0.06673716,
                    -0.011297725,
                    0.026882315,
                    -0.0271604,
                    0.0053434824,
                    -0.04982204,
                    -0.014138874,
                    -0.017344888,
                    0.023262324,
                    -0.027566284,
                    0.028769746,
                    -0.01790407,
                    0.021286923,
                    0.037220098,
                    0.027963053,
                    0.018125042,
                    -0.024682406,
                    -0.015903063,
                    0.008201522,
                    0.047491897,
                    0.04206297,
                    -0.02044235,
                    -0.026820127,
                    0.017688768,
                    -0.038202174,
                    0.04803243,
                    -0.020497348,
                    -0.06809881,
                    0.019102287,
                    -0.016282575,
                    -0.008147121,
                    0.023472652,
                    0.066234715,
                    0.029089477,
                    -0.05138362,
                    -0.0050375434,
                    -0.025048234,
                    -0.027642297,
                    -0.0064551206,
                    -0.03406217,
                    -0.0003015566,
                    0.01686922,
                    0.051587597,
                    -0.05335174,
                    -0.015375454,
                    -0.006774749,
                    0.017535774,
                    0.0075883474,
                    -0.027056802,
                    -0.010513732,
                    -0.10870002,
                    0.038311325,
                    -0.023443738,
                    0.0006942114,
                    0.0017571698,
                    0.04022458,
                    0.0053058867,
                    -0.013459642,
                    0.037681907,
                    -0.031173995,
                    -0.019205242,
                    0.046206266,
                    0.0021427372,
                    0.026726939,
                    0.07782731,
                    0.037151683,
                    0.079537086,
                    -0.011208424,
                    0.046385463,
                    -0.04847148,
                    -0.044214733,
                    -0.024537982,
                    0.08815229,
                    -0.0019893285,
                    0.02862184,
                    0.04221302,
                    0.051742513,
                    -0.040929493,
                    -0.025749704,
                    0.015830496,
                    0.023573006,
                    0.107543774,
                    0.020702802,
                    -0.007434051,
                    0.013178751,
                    -0.014609835,
                    0.0055954396,
                    0.055712786,
                    -0.07954713,
                    0.00041368845,
                    -0.075608104,
                    -0.007885142,
                    -0.010423114,
                    -0.018631916,
                    0.027715273,
                    -0.04978383,
                    -0.050067883,
                    -0.079438716,
                    0.014000716,
                    0.030478386,
                    0.009050721,
                    -0.025198627,
                    -0.0028137376,
                    0.06723492,
                    -0.048240956,
                    0.017409516,
                    0.018344734,
                    0.007830445,
                    -0.02456571,
                    -0.0030882207,
                    -0.0051813135,
                    -0.035331354,
                    0.029751945,
                    -0.044173956,
                    0.0015829265,
                    0.012597751,
                    0.030796584,
                    -0.03777858,
                    0.072733246,
                    0.06821187,
                    0.024542531,
                    0.06631259,
                    -0.03229986,
                    -0.04381717,
                    -0.106004834,
                    -0.0011497795,
                    -0.00930112,
                    -0.029783485,
                    0.03572324,
                    0.042525727,
                    -0.054525614,
                    -0.01470237,
                    0.00053973304,
                    -0.0041476768,
                    -0.022199042,
                    0.02284123,
                    -0.02174277,
                    0.011732314,
                    0.057626083,
                    -0.07894061,
                    0.09304443,
                    -0.005694812,
                    0.0051541748,
                    -0.049611423,
                    0.037705578,
                    -0.025591472,
                    -0.021194516,
                    -0.027488453,
                    0.049624503,
                    -0.070989944,
                    0.01657545,
                    0.013350953,
                    0.00841555,
                    0.013943526,
                    -0.032539055,
                    0.0014774579,
                    -0.04796844,
                    -0.015522849,
                    -0.037159007,
                    -0.022781903,
                    0.005494773,
                    0.00034274854,
                    0.03895854,
                    0.040065814,
                    0.031668853,
                    -0.10396242,
                    -0.060128905,
                    -0.006456444,
                    -0.0012389452,
                    0.015883658,
                    0.04947868,
                    0.082508236,
                    -0.027179895,
                    0.02871063,
                    0.024578592,
                    -0.009762573,
                    0.007507379,
                    0.026203629,
                    0.016447682,
                    -0.03294233,
                    -0.051270362,
                    0.023154914,
                    0.013433654,
                    -0.030371645,
                    0.024437387,
                    -0.011202293,
                    -0.000749408,
                    0.027105926,
                    -0.0388552,
                    -0.056164227,
                    0.050209496,
                    0.010824088,
                    -0.032063395,
                    -0.054537274,
                    0.012184584,
                    -0.0067816237,
                    0.018609297,
                    -0.0139637,
                    0.02463699,
                    -0.0046339706,
                    0.06117317,
                    -0.04965085,
                    -0.031187585,
                    0.011864996,
                    -0.0012917097,
                    0.060485847,
                    0.035890013,
                    0.0068307775,
                    0.0043460564,
                    -0.0019570603,
                    -0.027023794,
                    -0.007598705,
                    0.006394645,
                    0.0058349767,
                    0.042051516,
                    -0.023572199,
                    -0.033351462,
                    0.031145293,
                    -0.0120965075,
                    -0.033105936,
                    -0.04605126,
                    -0.056910872,
                    -0.005600399,
                    0.017727192,
                    -0.02330314,
                    0.011734753,
                    0.047223542,
                    0.03492995,
                    0.04730999,
                    0.050469574,
                    0.0036776129,
                    -0.020731065,
                    -0.05031616,
                    -0.040163178,
                    -0.08734381,
                    -0.052698817,
                    0.00123459,
                    0.010232824,
                    0.025452584,
                    -0.0026405696,
                    -0.0037284044,
                    0.018655455,
                    0.06219949,
                    -0.017533502,
                    -0.0053410535,
                    -0.07754048,
                    -0.050167803,
                    -0.0760839,
                    -0.015050281,
                    0.011657816,
                    0.056118097,
                    -0.07960886,
                    0.017400416,
                    -0.037782155,
                    -0.0134981265,
                    -0.014786434,
                    0.01310445,
                    0.067695916,
                    -0.002906271,
                    0.012523397,
                    -0.06826953,
                    -0.021396834,
                    0.026826175,
                    0.040257465,
                    -0.007992264,
                    -0.046851274,
                    -0.052621055,
                    -0.059978846,
                    -0.0076254276,
                    -0.007630302,
                    -0.028449025,
                    -0.018142525,
                    0.040817857,
                    0.020701878,
                    0.00365317,
                    -0.051722288,
                    0.022785434,
                    -0.0126320375,
                    0.0367689,
                    -0.0016271791,
                    0.026618771,
                    -0.0023710267,
                    0.014240093,
                    0.055501103,
                    -0.031106712,
                    0.013023728,
                    0.029487276,
                    0.018924234,
                    0.005408001,
                    -0.07983805,
                    0.0005971905,
                    0.01940745,
                    0.015026314,
                    0.02359795,
                    0.0054673213,
                    -0.07791104,
                    -0.048737913,
                    -0.064455554,
                    -0.12638979,
                    0.0008822773,
                    -0.07627577,
                    -0.0061467024,
                    -0.0008589217,
                    -0.028389007,
                    -0.041662283,
                    -0.021177875,
                    0.04416939,
                    -0.0036251508,
                    -0.013347197,
                    0.020907475,
                    -0.023388114,
                    0.005763718,
                    0.022672728,
                    0.010382343,
                    -0.0632251,
                    -0.04539014,
                    -0.0002913326,
                    -0.03358886,
                    -0.011878241,
                    0.00057383516,
                    0.045176484,
                    0.0044332715,
                    0.009115762,
                    -0.009695122,
                    0.04399701,
                    0.002461479,
                    0.0063320496,
                    -0.038243365,
                    0.014773573,
                    -0.041580588,
                    0.025570251,
                    0.00024998098,
                    0.022766402,
                    0.002696928,
                    0.042013437,
                    0.020654615,
                    -0.022486083,
                    0.020701941,
                    0.06769541,
                    -0.027665107,
                    0.025065677,
                    -0.058677126,
                    -0.005606738,
                    -0.030660594,
                    0.036190093,
                    0.005217673,
                    0.0038946,
                    -0.049907424,
                    0.018475268,
                    0.030534435,
                    -0.033634003,
                    -0.026514698,
                    0.061727878,
                    0.009071,
                    -0.020185351,
                    0.002289446,
                    -0.018585907,
                    -0.0067302696,
                    -0.012673388,
                    0.04209287,
                    0.05002671,
                    -0.044933718,
                    -0.010536026,
                    -0.022743523,
                    -0.0019238136,
                    -0.0060753403,
                    -0.022139898,
                    0.047112066,
                    -0.016026238,
                    0.025906768,
                    0.019289989,
                    0.021619951,
                    -0.012293178,
                    -0.024648042,
                    0.049239427,
                    0.033746533,
                    0.010492671,
                    -0.029039891,
                    -0.049868416,
                    0.033278476,
                    -0.004040017,
                    0.059626482,
                    -0.024160884,
                    -0.07498226,
                    0.039521437,
                    0.011464214,
                    -0.018192973,
                    -0.007372519,
                    0.043203346,
                    0.013612146,
                    -0.055734996,
                    -0.020606652,
                    0.022268532,
                    0.0025279694,
                    -0.0984952,
                    0.032667574,
                    -0.06070344,
                    0.029531656,
                    0.020999247,
                    0.01043111,
                    -0.03483265,
                    -0.004983468,
                    0.009015729,
                    0.013509287,
                    -0.016383434,
                    0.050168797,
                    0.043268766,
                    -0.07614634,
                    -0.02804231,
                    0.0032657727,
                    0.044405706,
                    -0.04485564,
                    0.007100709,
                    0.014508236,
                    -0.0069139157,
                    0.004806404,
                    0.017834296,
                    0.0074579506,
                    -0.0012751953,
                    -0.020872502,
                    -0.030439794,
                    -0.023535244,
                    -0.004779637,
                    -0.012195425,
                    0.02046227,
                    0.01699531,
                    0.0080058975,
                    0.01996157,
                    -0.029094048,
                    -0.013285501,
                    -0.008434056,
                    -0.0021355576,
                    -0.0038001782,
                    -0.006549041,
                    -0.05960305,
                    -0.032245938,
                    -0.08058905,
                    0.02066991,
                    0.0017652615,
                    0.002863885,
                    0.004771491,
                    0.03134235,
                    0.0019840382,
                    0.008926352,
                    0.054349653,
                    0.0126468595,
                    0.047189254,
                    -0.045331728,
                    -0.0031931093,
                    0.05145703,
                    0.028904984,
                    0.038406808,
                    0.076466255,
                    -0.0082346415,
                    0.041650582,
                    0.00038202645,
                    -0.040065803,
                    -0.0034174302,
                    0.07883703,
                    -0.024085721,
                    0.015974889,
                    -0.014614687,
                    0.008826982,
                    0.013056139,
                    -0.036232814,
                    0.0017404106,
                    0.046567507,
                    -0.038627587,
                    -0.022772735,
                    -0.00360903,
                    0.06401064,
                    0.018614624,
                    0.024335671,
                    -0.01810877,
                    0.018877637,
                    0.0076220147,
                    0.027648995,
                    -0.02293826,
                    0.02607716,
                    0.0022417856,
                    0.035924125,
                    0.031712912,
                    0.015599443,
                    0.01184005,
                    -0.018949784,
                    -0.023008842,
                    0.017906023,
                    0.0064447913,
                    0.0022186881,
                    -0.023603898,
                    0.11304756,
                    0.013100798,
                    0.047045246,
                    -0.004973314,
                    0.037342075,
                    -0.037097525,
                    0.0064201783,
                    0.016215287,
                    -0.04781635,
                    0.04588995,
                    -0.04934734,
                    0.032746654,
                    -0.030598218,
                    -0.023398431,
                    -0.0009910475,
                    -0.012254112,
                    -0.013276868,
                    -0.049342964,
                    0.038995508,
                    -0.032407753,
                    0.02308919,
                    -0.008438302,
                    -0.0784327,
                    0.06612419,
                    0.0011994164,
                    0.04635904,
                    -0.018904384,
                    0.06974694,
                    0.01702173,
                    -0.01874133,
                    -0.029214252,
                    -0.020141654,
                    -0.0019626785,
                    0.018563015,
                    0.008967391,
                    0.026343763,
                    0.024692085,
                    0.012153884,
                    -0.03099715,
                    0.046586502,
                    0.0030605833,
                    0.037427466,
                    0.022576723,
                    -0.01451993,
                    -0.0053356336,
                    0.019817669,
                    -0.014174678,
                    -0.034082834,
                    -0.029313268,
                    0.028502822,
                    -0.053297944,
                    0.0057360632,
                    -0.009595574,
                    -0.012570278,
                    -0.049505513,
                    -0.06881855,
                    -0.019431435,
                    -0.020797614,
                    -0.010205221,
                    0.0212843,
                    0.0040956843,
                    -0.032163706,
                    -0.01579795,
                    0.011154651,
                    0.05165608,
                    -0.044125475,
                    0.051611453,
                    0.015573544,
                    -0.047152903,
                    -0.034506742,
                    0.042775963,
                    -0.0063938797,
                    0.022578692,
                    -0.000698978,
                    0.11097389,
                    0.0042039477,
                    -0.012744041,
                    0.025101813,
                    0.05587627,
                    -0.025829926,
                    -0.000377134,
                    0.018274203,
                    -0.044377174,
                    0.023601126,
                    -0.014993059,
                    -0.03146059,
                    -0.011369117,
                    -0.0068684905,
                    -0.047642265,
                    0.023028033,
                    0.05118852,
                    0.037967626,
                    0.03589492,
                    -0.034851607,
                    0.05836855,
                    0.007422612,
                    -0.0028261798,
                    -0.084631816,
                    -0.008416511,
                    -0.013297727,
                    0.045844845,
                    0.0017582007,
                    0.039870862,
                    0.038967937,
                    -0.049584012,
                    0.013567134,
                    -0.00683099,
                    0.01515794,
                    -0.020968834,
                    -0.046133906,
                    0.04676231,
                    -0.0072222347,
                    -0.027787503,
                    -0.0074175396,
                    0.021458842,
                    0.0051179165,
                    -0.024867475,
                    -0.0020387261,
                    0.0007908524,
                    0.031947404,
                    0.054386783,
                    0.003670804,
                    -0.029436346,
                    0.013517629,
                    0.03133852,
                    -0.04301945,
                    -0.012450221,
                    -0.011596534,
                    0.010312541,
                    -0.004814255,
                    0.013594315,
                    -0.00032178903,
                    -0.008096347,
                    0.010448234,
                    0.018018147,
                    0.012485202,
                    0.00015640758,
                    0.056029968,
                    -0.034684505,
                    -0.005945102,
                    0.04540071,
                    0.03652228,
                    0.017559635,
                    -0.0029428585,
                    0.031903584,
                    0.003917594,
                    -0.064691976,
                    -0.039297286,
                    0.010467662,
                    0.0093484605,
                    -0.014891123,
                    -0.009175688,
                    -0.04573362,
                    0.0035475963,
                    0.022246609,
                    0.03532225,
                    -0.017237678,
                    -0.034951076,
                    -0.023324898,
                    -0.027865818,
                    -0.010352668,
                    -0.029556517,
                    -0.01141666,
                    -0.023895623,
                    -0.008008368,
                    0.034369104,
                    -0.035263885,
                    -0.0023880412,
                    0.0037398383,
                    -0.043421715,
                    -0.026140437,
                    0.017505916,
                    0.01172007,
                    -0.0052012587,
                    -0.010666706,
                    -0.044013575,
                    -0.021390941,
                    0.00925609,
                    0.04468026,
                    -0.05654946,
                    0.05758538,
                    0.038788065,
                    -0.06254394,
                    0.033485573,
                    -0.03277277,
                    0.041688804,
                    -0.016151179,
                    0.029675728,
                    -0.052232865,
                    0.012067895,
                    -0.039755195,
                    -0.038339008,
                    -0.013288515,
                    0.009655733,
                    -0.015840251,
                    -0.017354984,
                    -0.00071384717,
                    -0.043791853,
                    -0.009513481,
                    0.023261022,
                    -0.04977938,
                    0.01587062,
                    -0.019334927,
                    -0.0021081585,
                    0.06377285,
                    0.026044562,
                    -0.004681057,
                    0.02466819,
                    0.0100608235,
                    -0.027291354,
                    0.019331526,
                    -0.017899454,
                    -0.036263637,
                    0.0022282766,
                    0.05268942,
                    0.03951839,
                    -0.018724138,
                    -0.01991192,
                    -0.037556514,
                    0.056540187,
                    -0.052383814,
                    0.041265804,
                    0.059506774,
                    0.025743607,
                    -0.045918416,
                    0.05303863,
                    -0.013928889,
                    -0.061523702,
                    0.02737446,
                    0.016407,
                    -0.03208383,
                    0.047133777,
                    0.020823127,
                    -0.049983423,
                    -0.094918594,
                    -0.039521795,
                    0.021127349,
                    0.0046334304,
                    -0.088479534,
                    -0.06331032,
                    -0.034863796,
                    -0.0043010265,
                    -0.09269081,
                    0.019175887,
                    0.012120077,
                    0.02139774,
                    0.038447183,
                    -0.06730395,
                    -0.0045507355,
                    -0.009664997,
                    0.032972448,
                    -0.025034687,
                    -0.0015198331,
                    0.082746305,
                    0.0031048357,
                    -0.009439897,
                    -0.113036305,
                    -0.076955914,
                    0.042393137,
                    -0.040755555,
                ]
            }
        },
    ],
}


@pytest.fixture(scope="session")
def simple_get(extract_shortened_prompt):
    def _simple_get(self):
        content_len = int(self.headers.get("content-length"))
        content = json.loads(self.rfile.read(content_len).decode("utf-8"))

        prompt = extract_shortened_prompt(content)
        if not prompt:
            self.send_response(500)
            self.end_headers()
            self.wfile.write("Could not parse prompt.".encode("utf-8"))
            return

        headers, response = ({}, "")

        for k, v in RESPONSES.items():
            if prompt.startswith(k):
                headers, status_code, response = v
                break
        else:  # If no matches found
            self.send_response(500)
            self.end_headers()
            self.wfile.write(("Unknown Prompt:\n%s" % prompt).encode("utf-8"))
            return

        # Send response code
        self.send_response(status_code)

        # Send headers
        for k, v in headers.items():
            self.send_header(k, v)
        self.end_headers()

        # Send response body
        self.wfile.write(json.dumps(response).encode("utf-8"))
        return

    return _simple_get


@pytest.fixture(scope="session")
def extract_shortened_prompt():
    def _extract_shortened_prompt(content):
        try:
            if "requests" in content.keys():
                prompt = content.get("requests")[0].get("content").get("parts")[0].get("text")
            else:
                prompt = content.get("contents")[0].get("parts")[0].get("text")

            prompt = prompt.lstrip().split("\n")[0]
        except Exception:
            prompt = ""
        return prompt

    return _extract_shortened_prompt


@pytest.fixture(scope="session")
def MockExternalGeminiServer(simple_get):
    class _MockExternalGeminiServer(MockExternalHTTPServer):
        # To use this class in a test one needs to start and stop this server
        # before and after making requests to the test app that makes the external
        # calls.

        def __init__(self, handler=simple_get, port=None, *args, **kwargs):
            super(_MockExternalGeminiServer, self).__init__(handler=handler, port=port, *args, **kwargs)  # noqa: B026

    return _MockExternalGeminiServer


if __name__ == "__main__":
    with MockExternalGeminiServer() as server:
        print("MockExternalGeminiServer serving on port %s" % str(server.port))
        while True:
            pass  # Serve forever
